"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const browserify = require("browserify");
const browserifyIncremental = require("browserify-incremental");
const fs_1 = require("fs");
const path_1 = require("path");
const streamToString = require("stream-to-string");
const UglifyJS = require("uglify-es");
const logger_1 = require("../logger");
const entry = path_1.join('.seagull', 'node_modules', '@seagull-js', 'seagull', 'dist', 'lib', 'spa', 'entry.js');
class Bundler {
    constructor(optimize = false) {
        this.minify = false;
        this.sourceMaps = false;
        this.incrementalCache = {};
        this.incrementalPackageCache = {};
        this.minify = optimize;
        this.sourceMaps = !optimize;
        const browserifyArgs = {
            cache: this.incrementalCache,
            debug: this.sourceMaps,
            fullPaths: true,
            ignoreMissing: true,
            packageCache: this.incrementalPackageCache,
        };
        this.browserify = browserify(browserifyArgs);
        this.browserifyIncremental = browserifyIncremental(this.browserify);
        this.browserify.add(path_1.join(process.cwd(), entry));
        this.browserify.on('time', time => {
            logger_1.log(' Bundling took(ms):', time);
        });
    }
    static bundle(minify = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const bfy = browserify({ ignoreMissing: true });
            const stream = bfy.add(path_1.join(process.cwd(), entry)).bundle();
            const bundle = minify
                ? UglifyJS.minify(yield streamToString(stream)).code
                : yield streamToString(stream);
            const dist = path_1.join(process.cwd(), '.seagull', 'assets', 'bundle.js');
            fs_1.writeFileSync(dist, bundle, { encoding: 'utf-8' });
        });
    }
    bundle() {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = this.browserify.bundle();
            const bundle = this.minify
                ? UglifyJS.minify(yield streamToString(stream)).code
                : yield streamToString(stream);
            const dist = path_1.join(process.cwd(), '.seagull', 'assets', 'bundle.js');
            fs_1.writeFileSync(dist, bundle, { encoding: 'utf-8' });
            return bundle;
        });
    }
}
exports.Bundler = Bundler;
//# sourceMappingURL=bundler.js.map