"use strict";
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r);  }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const lodash_1 = require("lodash");
const path_1 = require("path");
const shell = require("shelljs");
const ts = require("typescript");
const logger_1 = require("../logger");
const helper_1 = require("./helper");
Symbol.asyncIterator =
    Symbol.asyncIterator || Symbol.for('Symbol.asyncIterator');
class Compiler {
    constructor() {
        this.tscFileWatcher = [];
        this.counter = 0;
        this.watch = function () {
            return __asyncGenerator(this, arguments, function* () {
                this.counter = 1;
                this.tsc = ts.createWatchProgram(this.host);
                while (true) {
                    yield __await(this.wait.compile);
                    this.createCompilePromise();
                    yield;
                }
            });
        };
        const tsConfig = JSON.parse(fs_1.readFileSync(path_1.join(process.cwd(), 'tsconfig.json'), 'utf-8'));
        this.conf = ts.parseJsonConfigFileContent(tsConfig, this.getTsSys(), process.cwd());
        this.conf.options.diagnostics = true;
        this.conf.options.extendedDiagnostics = true;
        this.host = ts.createWatchCompilerHost(this.conf.fileNames, this.conf.options, this.getTsSys(), undefined);
        this.host.trace = () => {
            return;
        };
        this.host.onWatchStatusChange = this.onWatchStatusChange.bind(this);
        this.host.afterProgramCreate = this.onCompilerMessage.bind(this);
        this.watchedFiles = this.conf.fileNames;
        this.createCompilePromise();
    }
    static compile() {
        shell.config.fatal = true;
        shell.exec(`${helper_1.binPath('tsc')}`);
    }
    stop() {
        this.tscFileWatcher.forEach(w => w.close());
        this.tsc.updateRootFileNames([]);
        this.host.trace = () => {
            return;
        };
        this.host.onWatchStatusChange = () => {
            return;
        };
        this.host.afterProgramCreate = () => {
            return;
        };
    }
    getTsSys() {
        const sys = lodash_1.cloneDeep(ts.sys);
        sys.watchFile = (path, callback, pollingInterval) => {
            const wrappedCB = (fileName, eventKind) => {
                if (!this.isInWatchedDir(fileName)) {
                    return callback(fileName, eventKind);
                }
                if (eventKind === 1) {
                    this.changedWatchedFile(fileName);
                }
                if (eventKind === 2) {
                    this.deletedWatchedFile(fileName);
                }
                callback(fileName, eventKind);
            };
            const watcher = ts.sys.watchFile(path, wrappedCB, pollingInterval);
            this.tscFileWatcher.push(watcher);
            return watcher;
        };
        sys.watchDirectory = (path, callback, recursive) => {
            const wrappedCB = fileName => {
                if (this.isInWatchedDir(fileName) &&
                    fs_1.existsSync(fileName) &&
                    fs_1.lstatSync(fileName).isFile()) {
                    this.addWatchedFile(fileName);
                }
                return callback(fileName);
            };
            const watcher = ts.sys.watchDirectory(path, wrappedCB, recursive);
            this.tscFileWatcher.push(watcher);
            return watcher;
        };
        return sys;
    }
    isInWatchedDir(path) {
        return Object.keys(this.conf.wildcardDirectories).reduce((acc, wDir) => acc ? !!acc : path.toLowerCase().startsWith(wDir.toLowerCase()), false);
    }
    createCompilePromise() {
        this.wait = {};
        this.wait.compile = new Promise(resolve => (this.wait.resolve = resolve));
    }
    changedWatchedFile(filePath) {
        this.counter++;
        logger_1.log('Watched file changed:', filePath);
    }
    addWatchedFile(filePath) {
        if (!this.tsc || this.watchedFiles.indexOf(filePath) > -1) {
            return;
        }
        logger_1.log('Watching new file:', filePath);
        this.counter++;
        this.watchedFiles = this.watchedFiles.concat(filePath);
        this.tsc.updateRootFileNames(this.watchedFiles);
    }
    deletedWatchedFile(filePath) {
        this.watchedFiles = this.watchedFiles.filter(file => file !== filePath);
        this.tsc.updateRootFileNames(this.watchedFiles);
        this.counter++;
        let compiledFile = path_1.join('.seagull', 'dist', path_1.relative(process.cwd(), filePath));
        const ext = path_1.extname(compiledFile);
        compiledFile = compiledFile.replace(RegExp(`${ext}$`), '');
        const obseleteJs = compiledFile + '.js';
        const obseleteJsMap = compiledFile + '.js.map';
        if (fs_1.existsSync(obseleteJs)) {
            fs_1.unlinkSync(obseleteJs);
        }
        if (fs_1.existsSync(obseleteJsMap)) {
            fs_1.unlinkSync(obseleteJsMap);
        }
        logger_1.log('Removing file', filePath);
    }
    onWatchStatusChange(diagnostic, newline) {
        if (diagnostic.code !== 6042) {
            return;
        }
        if (this.counter === 1) {
            logger_1.log('Compile finished. Waiting for file changes');
        }
        if (this.counter > 0) {
            this.counter--;
            this.wait.resolve();
        }
    }
    onCompilerMessage(programInfo) {
        programInfo.emit();
        const diagnostics = programInfo.getSyntacticDiagnostics();
        diagnostics.forEach(diagnostic => {
            this.logDiagnostic(diagnostic);
        });
    }
    logDiagnostic(dg) {
        const message = ts.flattenDiagnosticMessageText(dg.messageText, '\n');
        if (!dg.file) {
            logger_1.log(`  Error: ${message}`);
            return;
        }
        const pos = dg.file.getLineAndCharacterOfPosition(dg.start);
        const positionDisplay = `${pos.line + 1},${pos.character + 1}`;
        logger_1.log(` Error ${dg.file.fileName} (${positionDisplay}): ${message}`);
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=compiler.js.map