"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const dir = require("node-dir");
const path_1 = require("path");
const shell = require("shelljs");
const app_1 = require("../loader/app");
const generate_yaml_1 = require("../serverless/generate-yaml");
function binPath(name) {
    return path_1.join(__dirname, '..', '..', '..', 'node_modules', '.bin', name);
}
exports.binPath = binPath;
function cleanBuildDirectory() {
    shell.rm('-rf', '.seagull');
}
exports.cleanBuildDirectory = cleanBuildDirectory;
function lint() {
    shell.exec(`${binPath('tslint')} -c tslint.json --fix src/**/*.ts`);
}
exports.lint = lint;
function prettier() {
    const args = '--single-quote --no-semi --trailing-comma es5';
    const cmd = `${binPath('prettier')} ${args} --write src/**/*.ts`;
    shell.exec(cmd, { silent: true });
}
exports.prettier = prettier;
function copyAssets() {
    if (!fs_1.existsSync(path_1.join('frontend', 'assets'))) {
        return;
    }
    const files = dir.files('frontend/assets', { sync: true });
    if (!files || files.length === 0) {
        return;
    }
    shell.cp('-R', 'frontend/assets/*', '.seagull/assets');
}
exports.copyAssets = copyAssets;
function initFolder() {
    if (fs_1.existsSync(path_1.join(shell.pwd().toString(), '.seagull'))) {
        cleanBuildDirectory();
    }
    shell.mkdir('-p', '.seagull');
    shell.cp('package.json', '.seagull/package.json');
    if (!fs_1.existsSync(path_1.join(shell.pwd().toString(), '.seagull', 'node_modules'))) {
        shell.ln('-s', '../node_modules', `./.seagull/node_modules`);
    }
    shell.mkdir('-p', '.seagull/assets');
}
exports.initFolder = initFolder;
function createServerlessYaml() {
    const pwd = shell.pwd().toString();
    const app = new app_1.default(pwd);
    const yml = generate_yaml_1.default(app);
    const ymlPath = path_1.join(pwd, '.seagull', 'serverless.yml');
    fs_1.writeFileSync(ymlPath, yml);
}
exports.createServerlessYaml = createServerlessYaml;
//# sourceMappingURL=helper.js.map