"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const dir = require("node-dir");
const path_1 = require("path");
const shell = require("shelljs");
function modifyScriptExports() {
    const files = dir
        .files('.seagull/dist/backend/api', { sync: true })
        .filter(file => /\.js$/.test(file));
    const from = /exports\.default = (\w+);/;
    const to = 'exports.default = $1;\nexports.handler = $1.dispatch.bind($1);';
    shell.sed('-i', from, to, files);
}
exports.modifyScriptExports = modifyScriptExports;
function addImportIndexFile() {
    const frontendDir = path_1.join('.seagull', 'dist', 'frontend');
    function listFiles(directory) {
        if (!fs_1.existsSync(path_1.join(frontendDir, directory))) {
            return [];
        }
        const files = dir
            .files(path_1.join(frontendDir, directory), {
            recursive: false,
            sync: true,
        })
            .filter(file => /\.js$/.test(file));
        return files || [];
    }
    function buildImportKeys(files) {
        return files
            .map(file => {
            const key = file
                .replace(/\.js$/, '')
                .split('/')
                .reverse()[0];
            return `"${key}":require("${file.replace(frontendDir, '.')}")`;
        })
            .join(',\n');
    }
    const stores = listFiles('stores');
    const pages = listFiles('pages');
    const indexExport = `
    module.exports = {
      stores: {${buildImportKeys(stores)}},
      pages: {${buildImportKeys(pages)}}
    }
  `;
    fs_1.writeFileSync(path_1.join(frontendDir, 'index.js'), indexExport);
}
exports.addImportIndexFile = addImportIndexFile;
//# sourceMappingURL=transforms.js.map