"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const lodash_1 = require("lodash");
const path_1 = require("path");
const shell = require("shelljs");
const Gen = require("./index");
class App {
    constructor(name) {
        this.name = name;
        const dir = {};
        dir['package.json'] = Gen.generatePackageJson(name);
        dir['tsconfig.json'] = Gen.generateTsconfigJson();
        dir['tslint.json'] = Gen.generateTslintJson();
        dir['backend/api/Frontend.ts'] = Gen.generateSsrApi();
        dir['frontend/layout.tsx'] = Gen.generateLayoutTsx(name);
        dir['frontend/pages/hello.tsx'] = Gen.generatePage('HelloPage', {
            path: '/',
        });
        this.dir = dir;
    }
    toFolder(path) {
        this.createFolderStructure(path);
        this.copyExampleAssetFile(path);
        lodash_1.toPairs(this.dir).forEach(([file, text]) => {
            fs_1.writeFileSync(path_1.join(path, file), text);
        });
    }
    createFolderStructure(path) {
        fs_1.mkdirSync(path);
        fs_1.mkdirSync(path_1.join(path, 'backend'));
        fs_1.mkdirSync(path_1.join(path, 'backend', 'api'));
        fs_1.mkdirSync(path_1.join(path, 'frontend'));
        fs_1.mkdirSync(path_1.join(path, 'frontend/pages'));
        fs_1.mkdirSync(path_1.join(path, 'frontend/assets'));
    }
    copyExampleAssetFile(path) {
        const srcFile = path_1.join(__dirname, '../../..', 'media', 'seagull-logo.png');
        const destFile = path_1.join(path, 'frontend', 'assets', 'seagull-logo.png');
        shell.cp(srcFile, destFile);
    }
}
exports.default = App;
//# sourceMappingURL=app.js.map