"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const prettier = require("prettier");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = require("typescript");
const astSettings = {
    manipulationSettings: {
        indentationText: ts_simple_ast_1.IndentationText.TwoSpaces,
        quoteType: ts_simple_ast_1.QuoteType.Single,
        scriptTarget: ts.ScriptTarget.Latest,
    },
    useVirtualFileSystem: true,
};
const prettierSettings = {
    semi: false,
    singleQuote: true,
    trailingComma: 'es5',
};
class Base {
    constructor() {
        const ast = new ts_simple_ast_1.default(astSettings);
        this.sourceFile = ast.createSourceFile('virtual.ts');
    }
    addDefaultImport(from, name, legacy) {
        const token = legacy ? `* as ${name}` : name;
        const opts = { moduleSpecifier: from, defaultImport: token };
        this.sourceFile.addImportDeclaration(opts);
        return this;
    }
    addNamedImports(from, names) {
        const namedImports = names.map(name => ({ name }));
        const opts = { moduleSpecifier: from, namedImports };
        this.sourceFile.addImportDeclaration(opts);
        return this;
    }
    addInterface(name) {
        const intf = { name, export: true };
        this.sourceFile.addInterface(intf);
        return this;
    }
    toString() {
        const sourceCode = this.sourceFile.getFullText();
        const prettyCode = prettier.format(sourceCode, prettierSettings);
        return prettyCode;
    }
    toFile(filePath) {
        fs_1.writeFileSync(filePath, this.toString(), 'utf-8');
    }
}
exports.default = Base;
//# sourceMappingURL=base.js.map