"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("./base");
class default_1 extends base_1.default {
    constructor(name, parent, addDefaultInterfaces) {
        super();
        if (addDefaultInterfaces) {
            this.addInterface('IProps');
            this.addInterface('IState');
        }
        this.classDeclaration = this.sourceFile.addClass({ name });
        this.classDeclaration.setIsDefaultExport(true);
        if (parent) {
            this.classDeclaration.setExtends(parent);
        }
    }
    addConstructor(params) {
        this.classDeclaration.addConstructor(params);
        return this;
    }
    addMethod(params) {
        const opts = { isStatic: !!params.static, name: params.name };
        const method = this.classDeclaration.addMethod(opts);
        if (params.async) {
            method.setIsAsync(true);
        }
        if (params.parameter && params.parameter.length) {
            params.parameter.forEach((p, i) => method.insertParameter(i, p));
        }
        if (params.returnType) {
            method.setReturnType(params.returnType);
        }
        if (params.body) {
            method.setBodyText(params.body);
        }
        if (params.doc) {
            method.addJsDoc({ description: params.doc });
        }
        return this;
    }
    addProp(params) {
        const { name, type } = params;
        const opts = { name, type, isStatic: !!params.static };
        const prop = this.classDeclaration.addProperty(opts);
        if (params.value) {
            prop.setInitializer(params.value);
        }
        if (params.decorators && params.decorators.length) {
            prop.addDecorators(params.decorators);
        }
        if (params.doc) {
            prop.addJsDoc({ description: params.doc });
        }
        return this;
    }
}
exports.default = default_1;
//# sourceMappingURL=class.js.map