"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const class_1 = require("./class");
function generateAPI(name, options) {
    const gen = new class_1.default(name, 'API');
    gen.addNamedImports('@seagull-js/seagull', ['API', 'Request', 'Response']);
    if (options.path) {
        const docPath = `The URL path where this API will be located. Skip for private functions like cronjobs. Example: '/greetings/{name}'`;
        gen.addProp({
            doc: docPath,
            name: 'path',
            static: true,
            type: 'string',
            value: `'${options.path}'`,
        });
    }
    if (options.method || options.path) {
        const docMethod = `This is the HTTP method / verb for the API. Defaults to 'GET'`;
        gen.addProp({
            doc: docMethod,
            name: 'method',
            static: true,
            type: 'string',
            value: `'${options.method || 'GET'}'`,
        });
    }
    if (options.cors) {
        const docCors = `The URL path where this API will be located. Skip for private functions like cronjobs. Example: '/greetings/{name}'`;
        gen.addProp({
            doc: docCors,
            name: 'cors',
            static: true,
            type: 'boolean',
            value: `${!!options.cors}`,
        });
    }
    const docHandle = `This handle function executes your code. Return one of the following method invocations: 'text', 'json', 'redirect', 'missing', 'error'`;
    gen.addMethod({
        async: true,
        body: options.body || `return this.text('hello world')`,
        doc: docHandle,
        name: 'handle',
        parameter: [{ name: 'request', type: 'Request' }],
        returnType: 'Promise<Response>',
    });
    return gen;
}
exports.default = generateAPI;
//# sourceMappingURL=generate_api.js.map