"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const seagull_1 = require("@seagull-js/seagull");
const bodyParser = require("body-parser");
const express = require("express");
const path_1 = require("path");
const stoppable = require("stoppable");
const logger_1 = require("../logger");
class Server {
    constructor() {
        this.server = express();
        this.server.use((req, res, next) => {
            res.setHeader('Cache-Control', 'no-cache, no-store');
            next();
        });
        this.server.use(bodyParser.urlencoded({ extended: false }));
        this.server.use(bodyParser.json());
        const assetFolder = path_1.join(process.cwd(), '.seagull', 'assets');
        this.server.use('/assets', express.static(assetFolder, { maxAge: '0' }));
        this.server.use((req, resp, next) => {
            if (!this.router) {
                return;
            }
            this.router(req, resp, next);
        });
        this.server = stoppable(this.server, 0);
    }
    start(port = 3000) {
        if (process.env.NODE_ENV === 'test') {
            return this.server.listen(port, () => logger_1.log(`static server ready on localhost:${port}`));
        }
        else {
            this.server.listen(port, () => logger_1.log(`static server ready on localhost:${port}`));
        }
    }
    loadApp(app) {
        this.router = express.Router();
        for (const api of app.backend) {
            const fn = (req, res) => __awaiter(this, void 0, void 0, function* () {
                const request = mapRequestFormat(req);
                const handler = new api.module();
                const response = yield handler.handle(request);
                res.status(response.statusCode);
                res.setHeader('Content-Type', response.headers['Content-Type']);
                res.send(response.body);
            });
            const method = api.module.method.toString().toLowerCase();
            this.router[method](api.module.path.replace('/*', '*').toString(), fn);
        }
    }
}
exports.Server = Server;
function mapRequestFormat(req) {
    const method = req.method;
    return new seagull_1.Request(method, req.path, req.query, req.body);
}
//# sourceMappingURL=index.js.map