"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const YAML = require("yamljs");
class Builder {
    constructor(name) {
        this.data = this.createDefaultServerless();
        this.data.service = name;
    }
    addIAMRoleStatement(item) {
        this.data.provider.iamRoleStatements.push(item);
        return this;
    }
    addPlugin(item) {
        this.data.plugins.push(item);
        return this;
    }
    addFunction(name, fn) {
        this.data.functions[name] = fn;
        return this;
    }
    addTable(name, table) {
        this.data.resources.Resources[name] = table;
        return this;
    }
    addDistribution(name, distribution) {
        this.data.resources.Resources[name] = distribution;
        return this;
    }
    addS3Bucket(name, bucket) {
        this.data.resources.Resources[name] = bucket;
        return this;
    }
    addS3BucketPermission(name, bucketPermission) {
        this.data.resources.Resources[name] = bucketPermission;
        return this;
    }
    addDistributionAccessIdentity(name, accessIdentity) {
        this.data.resources.Resources[name] = accessIdentity;
        return this;
    }
    toYAML() {
        const json = JSON.parse(JSON.stringify(this.data));
        return YAML.stringify(json, 42, 2);
    }
    createDefaultServerless() {
        return {
            frameworkVersion: '=1.19.0',
            functions: {},
            plugins: [],
            provider: this.createDefaultProvider(),
            resources: { Resources: {} },
            service: '',
        };
    }
    createDefaultProvider() {
        return {
            iamRoleStatements: [],
            name: 'aws',
            region: 'eu-central-1',
            runtime: 'nodejs6.10',
            stage: 'dev',
            timeout: 30,
        };
    }
}
exports.default = Builder;
//# sourceMappingURL=builder.js.map