"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
class Distribution {
    constructor(opt) {
        this.apiService = opt.apiService;
        this.alias = opt.alias;
        this.targets = opt.targets;
        this.web = opt.web;
    }
    get customErrorResponses() {
        const responses = lodash_1.get(this, 'web.responses', {});
        return lodash_1.map(responses, (response, srcErrCode) => {
            return {
                ErrorCachingMinTTL: response.ttl || 0,
                ErrorCode: srcErrCode,
                ResponseCode: response.code,
                ResponsePagePath: response.path,
            };
        });
    }
    get viewerCertificate() {
        const certACM = lodash_1.get(this, `alias.cert`, false);
        if (!certACM) {
            return undefined;
        }
        return {
            AcmCertificateArn: certACM,
            MinimumProtocolVersion: 'TLSv1.1_2016',
            SslSupportMethod: 'sni-only',
        };
    }
    json() {
        const origins = this.targets.map(target => target.config());
        const endpoints = lodash_1.flatten(this.targets.map(target => target.endpoints()));
        const defaultCacheBehavior = lodash_1.head(this.targets).defaultEndpoint();
        const cacheBehaviors = endpoints.map(endpoint => endpoint.json());
        return {
            Properties: {
                DistributionConfig: {
                    Aliases: lodash_1.get(this, `alias.domains`, undefined),
                    CacheBehaviors: cacheBehaviors,
                    Comment: `${this.apiService}`,
                    CustomErrorResponses: this.customErrorResponses,
                    DefaultCacheBehavior: defaultCacheBehavior.json(),
                    DefaultRootObject: lodash_1.get(this, `web.default`, undefined),
                    Enabled: true,
                    HttpVersion: 'http2',
                    Origins: origins,
                    ViewerCertificate: this.viewerCertificate,
                },
            },
            Type: 'AWS::CloudFront::Distribution',
        };
    }
}
exports.default = Distribution;
//# sourceMappingURL=index.js.map