"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const fs_1 = require("fs");
const mocha_typescript_1 = require("mocha-typescript");
const path_1 = require("path");
const bundler_1 = require("../../../lib/build/bundler");
const compiler_1 = require("../../../lib/build/compiler");
const transforms_1 = require("../../../lib/build/transforms");
const functional_test_1 = require("../../helper/functional_test");
const shell = require("shelljs");
function compileAndModifySteps() {
    compiler_1.Compiler.compile();
    transforms_1.modifyScriptExports();
    transforms_1.addImportIndexFile();
}
let CompilerBuilderTest = class CompilerBuilderTest extends functional_test_1.default {
    constructor() {
        super(...arguments);
        this.buildDir = path_1.join(this.appDir, '.seagull');
        this.bundleDir = path_1.join(this.buildDir, 'assets');
        this.bundlePath = path_1.join(this.bundleDir, 'bundle.js');
    }
    before() {
        process.chdir(this.appDir);
        shell.mkdir('-p', this.bundleDir);
        if (!fs_1.existsSync(path_1.join(this.appDir, '.seagull', 'node_modules'))) {
            shell.ln('-s', '../node_modules', `./.seagull/node_modules`);
        }
    }
    after() {
        process.chdir(path_1.join(this.appDir, '..'));
        if (!fs_1.existsSync(this.bundlePath)) {
            return;
        }
        fs_1.unlinkSync(this.bundlePath);
    }
    'can bundle a project staticly'() {
        return __awaiter(this, void 0, void 0, function* () {
            const entry = path_1.join(process.cwd(), '.seagull', 'node_modules', '@seagull-js', 'seagull', 'dist', 'lib', 'spa', 'entry.js');
            this.addPage('SomePage', { path: '/some_url' });
            compileAndModifySteps();
            chai_1.expect(fs_1.existsSync(this.bundlePath)).to.be.equal(false);
            yield bundler_1.Bundler.bundle();
            chai_1.expect(fs_1.existsSync(this.bundlePath)).to.be.equal(true);
        });
    }
    'can bundle a project dynamicly'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addPage('SomePage', { path: '/some_url' });
            compileAndModifySteps();
            const bundler = new bundler_1.Bundler();
            chai_1.expect(fs_1.existsSync(this.bundlePath)).to.be.equal(false);
            yield bundler.bundle();
            chai_1.expect(fs_1.existsSync(this.bundlePath)).to.be.equal(true);
        });
    }
    'can bundle an added file dynamicly'() {
        return __awaiter(this, void 0, void 0, function* () {
            compiler_1.Compiler.compile();
            const bundler = new bundler_1.Bundler();
            let bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('some_added_url') > -1).to.be.false;
            this.addPage('SomePage', { path: '/some_added_url' });
            compileAndModifySteps();
            bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('some_added_url') > -1).to.be.true;
            chai_1.expect(fs_1.existsSync(this.bundlePath)).to.be.equal(true);
        });
    }
    'can bundle a changed file dynamicly'() {
        return __awaiter(this, void 0, void 0, function* () {
            const bundler = new bundler_1.Bundler();
            this.addPage('ChangedFile', { path: '/some_changed' });
            compileAndModifySteps();
            let bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('some_changed') > -1).to.be.true;
            this.addPage('ChangedFile', { path: '/changed_some' });
            compileAndModifySteps();
            bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('some_changed') > -1).to.be.false;
            chai_1.expect(bundle.indexOf('changed_some') > -1).to.be.true;
        });
    }
    'can remove a deleted file from bundle'() {
        return __awaiter(this, void 0, void 0, function* () {
            const bundler = new bundler_1.Bundler();
            this.addPage('DeletedFile', { path: '/deleted_string' });
            compileAndModifySteps();
            let bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('deleted_string') > -1).to.be.true;
            fs_1.unlinkSync(path_1.join(this.appDir, 'frontend', 'pages', 'DeletedFile.tsx'));
            fs_1.unlinkSync(path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'DeletedFile.js'));
            fs_1.unlinkSync(path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'DeletedFile.js.map'));
            compileAndModifySteps();
            bundle = yield bundler.bundle();
            chai_1.expect(bundle.indexOf('deleted_string') > -1).to.be.false;
        });
    }
};
__decorate([
    mocha_typescript_1.slow(5000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can bundle a project staticly", null);
__decorate([
    mocha_typescript_1.slow(5000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can bundle a project dynamicly", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can bundle an added file dynamicly", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can bundle a changed file dynamicly", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can remove a deleted file from bundle", null);
CompilerBuilderTest = __decorate([
    mocha_typescript_1.suite('builder::bundler')
], CompilerBuilderTest);
//# sourceMappingURL=bundler.js.map