"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator];
    return m ? m.call(o) : typeof __values === "function" ? __values(o) : o[Symbol.iterator]();
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const fs_1 = require("fs");
const mocha_typescript_1 = require("mocha-typescript");
const path_1 = require("path");
const shell = require("shelljs");
const villa_1 = require("villa");
const compiler_1 = require("../../../lib/build/compiler");
const functional_test_1 = require("../../helper/functional_test");
let CompilerBuilderTest = class CompilerBuilderTest extends functional_test_1.default {
    'can compile a file staticly'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addPage('SomePage', { path: '/some_url' });
            compiler_1.Compiler.compile();
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomePage.js');
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(true);
        });
    }
    'watching compiler notifies about finished initial compile'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addPage('SomePage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            try {
                for (var _a = __asyncValues(compiler.watch()), _b; _b = yield _a.next(), !_b.done;) {
                    const noMsg = yield _b.value;
                    chai_1.expect(true).equals(true);
                    break;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) yield _c.call(_a);
                }
                finally { if (e_1) throw e_1.error; }
            }
            compiler.stop();
            var e_1, _c;
        });
    }
    'watching compiler compiles added file'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'AddedFile.js');
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
            this.addPage('AddedFile', { path: '/some_url' });
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(true);
            compiler.stop();
        });
    }
    'watching compiler compiles multiple added files'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file1 = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'AddedFile1.js');
            const file2 = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'AddedFile2.js');
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file1)).to.be.equal(false);
            chai_1.expect(fs_1.existsSync(file2)).to.be.equal(false);
            this.addPage('AddedFile1', { path: '/some_url' });
            this.addPage('AddedFile2', { path: '/some_url' });
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file1)).to.be.equal(true);
            chai_1.expect(fs_1.existsSync(file2)).to.be.equal(true);
            compiler.stop();
        });
    }
    'stopped watching compiler does not compile added'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeStoppedAddedPage.js');
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            compiler.stop();
            this.addPage('SomeStoppedAddedPage', { path: '/some_url' });
            yield villa_1.sleep(500);
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
        });
    }
    'stopped watching compiler does not compile changed'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeStoppedChangedPage.js');
            const changedCode = 'const x = 3';
            this.addPage('SomeStoppedChangedPage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            compiler.stop();
            yield villa_1.sleep(100);
            fs_1.writeFileSync(path_1.join(this.appDir, 'frontend', 'pages', 'SomeStoppedChangedPage.tsx'), changedCode);
            yield villa_1.sleep(500);
            chai_1.expect(fs_1.readFileSync(file, 'utf8')).to.be.not.equal(changedCode);
        });
    }
    'watching compiler compiles changed file'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeChangedPage.js');
            const changedCode = 'const x = 3';
            this.addPage('SomeChangedPage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            fs_1.writeFileSync(path_1.join(this.appDir, 'frontend', 'pages', 'SomeChangedPage.tsx'), changedCode);
            yield compilationIterator.next();
            const newFileContent = fs_1.readFileSync(file, 'utf8');
            chai_1.expect(newFileContent.indexOf(changedCode) > -1).to.be.true;
            compiler.stop();
        });
    }
    'watching compiler compiles renamed file'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeRenamedPage.js');
            const renamed = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'Renamed.js');
            this.addPage('SomeRenamedPage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(true);
            fs_1.renameSync(path_1.join(this.appDir, 'frontend', 'pages', 'SomeRenamedPage.tsx'), 'Renamed.tsx');
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
            chai_1.expect(fs_1.existsSync(renamed)).to.be.equal(false);
            compiler.stop();
        });
    }
    'watching compiler removes deleted file'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeDeletedPage.js');
            this.addPage('SomeDeletedPage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            fs_1.unlinkSync(path_1.join(this.appDir, 'frontend', 'pages', 'SomeDeletedPage.tsx'));
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
            compiler.stop();
        });
    }
    'watching compiler removes deleted file in deleted folder'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pages', 'SomeDeletedPage.js');
            this.addPage('SomeDeletedPage', { path: '/some_url' });
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            shell.rm('-Rf', path_1.join(this.appDir, 'frontend', 'pages'));
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
            compiler.stop();
        });
    }
    'watching compiler compiles file in new folder'() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = path_1.join(this.appDir, '.seagull', 'dist', 'frontend', 'pager', 'newFolderFile.js');
            const compiler = new compiler_1.Compiler();
            const compilationIterator = compiler.watch();
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(false);
            fs_1.mkdirSync(path_1.join(this.appDir, 'frontend', 'pager'));
            fs_1.writeFileSync(path_1.join(this.appDir, 'frontend', 'pager', 'newFolderFile.ts'), 'const x = 3');
            yield compilationIterator.next();
            chai_1.expect(fs_1.existsSync(file)).to.be.equal(true);
            compiler.stop();
        });
    }
};
__decorate([
    mocha_typescript_1.slow(5000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "can compile a file staticly", null);
__decorate([
    mocha_typescript_1.slow(5000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler notifies about finished initial compile", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler compiles added file", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler compiles multiple added files", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "stopped watching compiler does not compile added", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "stopped watching compiler does not compile changed", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler compiles changed file", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler compiles renamed file", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler removes deleted file", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler removes deleted file in deleted folder", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CompilerBuilderTest.prototype, "watching compiler compiles file in new folder", null);
CompilerBuilderTest = __decorate([
    mocha_typescript_1.suite('builder::compiler')
], CompilerBuilderTest);
//# sourceMappingURL=compiler.js.map