"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const mocha_typescript_1 = require("mocha-typescript");
const fetch = require("node-fetch");
const path_1 = require("path");
const shell = require("shelljs");
const functional_test_1 = require("../../helper/functional_test");
let ServeCommandTest = class ServeCommandTest extends functional_test_1.default {
    constructor() {
        super(...arguments);
        this.server = null;
    }
    after() {
        if (this.server) {
            this.server.close();
        }
    }
    'can build a project'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addApi('hello', { path: '/hello' });
            this.addApi('responder', { path: '/responder' });
            const file = path_1.join(this.appDir, 'backend', 'api', 'responder.ts');
            const from = /return.+$/;
            const to = `return this.text('hello, ' + request.params.name)`;
            shell.sed('-i', from, to, file);
            yield this.build();
        });
    }
    'does load an app and starts the dev server'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve();
            const data = yield fetch('http://localhost:3000/hello');
            const text = yield data.text();
            chai_1.expect(text).to.be.equal('hello world');
        });
    }
    'can start dev server on arbitrary port'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve({ port: 4001 });
            const data = yield fetch('http://localhost:4001/hello');
            const text = yield data.text();
            chai_1.expect(text).to.be.equal('hello world');
        });
    }
    'does render html pages (SSR)'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve();
            const data = yield fetch('http://localhost:3000/');
            const html = yield data.text();
            chai_1.expect(html).to.include(`<title>${this.appName}</title>`);
        });
    }
    'does work with queryParameter'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve();
            const data = yield fetch('http://localhost:3000/responder?name=anon');
            const text = yield data.text();
            chai_1.expect(text).to.be.equal('hello, anon');
        });
    }
    'does generate bundle.js in-memory'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve();
            const data = yield fetch('http://localhost:3000/assets/bundle.js');
            const jsblob = yield data.text();
            chai_1.expect(jsblob).to.be.a('string');
            chai_1.expect(jsblob.length).to.be.above(1337);
        });
    }
    'does serve assets'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.server = yield this.serve();
            const data = yield fetch('http://localhost:3000/assets/seagull-logo.png');
            chai_1.expect(data.headers._headers['content-type']).to.be.deep.equal([
                'image/png',
            ]);
        });
    }
};
__decorate([
    mocha_typescript_1.timeout(40000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "can build a project", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "does load an app and starts the dev server", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "can start dev server on arbitrary port", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "does render html pages (SSR)", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "does work with queryParameter", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "does generate bundle.js in-memory", null);
__decorate([
    mocha_typescript_1.timeout(80000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "does serve assets", null);
ServeCommandTest = __decorate([
    mocha_typescript_1.suite('Commands::serve')
], ServeCommandTest);
//# sourceMappingURL=serve.js.map