"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const mocha_typescript_1 = require("mocha-typescript");
const YAML = require("yamljs");
const app_1 = require("../../../lib/loader/app");
const builder_1 = require("../../../lib/serverless/builder");
const generate_yaml_1 = require("../../../lib/serverless/generate-yaml");
const functional_test_1 = require("../../helper/functional_test");
let ServeCommandTest = class ServeCommandTest extends functional_test_1.default {
    'can build a project'() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addApi('Hello', { path: '/hello' });
            this.addModel('todos');
            yield this.build();
        });
    }
    'can generate a serverless.yml in memory'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.provider.name).to.be.equal('aws');
        chai_1.expect(yml.provider.runtime).to.be.equal('nodejs6.10');
        chai_1.expect(yml.provider.region).to.be.equal('eu-central-1');
        chai_1.expect(yml.provider.timeout).to.be.equal(30);
    }
    'yaml contains functions'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.functions).to.be.an('object');
        chai_1.expect(yml.functions).to.have.keys('api-Hello', 'api-Frontend');
    }
    'yaml functions have correct data fields'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        const fn = yml.functions['api-Hello'];
        chai_1.expect(fn.timeout).to.be.equal(30);
        chai_1.expect(fn.events).to.be.an('array');
        chai_1.expect(fn.handler).to.be.equal('dist/backend/api/Hello.handler');
    }
    'frontend function has correct paths (/ and /*)'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        const fn = yml.functions['api-Frontend'];
        chai_1.expect(fn.timeout).to.be.equal(30);
        chai_1.expect(fn.events).to.be.an('array');
        chai_1.expect(fn.events).to.have.length(2);
        chai_1.expect(fn.handler).to.be.equal('dist/backend/api/Frontend.handler');
    }
    'yaml contains tables'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.resources.Resources.todos).to.be.an('object');
        const table = yml.resources.Resources.todos;
        chai_1.expect(table.Type).to.be.equal('AWS::DynamoDB::Table');
    }
    'yaml contains cloudfront distribution'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.resources.Resources.distribution).to.be.an('object');
    }
    'yaml contains s3 bucket'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.resources.Resources.appBucket).to.be.an('object');
    }
    'yaml contains s3 permission'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.resources.Resources.appBucketPermission).to.be.an('object');
    }
    'parse yml does not contain null values for previously just undefined values'() {
        const app = new builder_1.default(undefined);
        const yml = YAML.parse(app.toYAML());
        chai_1.expect(yml.service).to.be.equal(undefined);
    }
    'yaml contains distribution access identity'() {
        const app = new app_1.default(this.appDir);
        const yml = YAML.parse(generate_yaml_1.default(app));
        chai_1.expect(yml.resources.Resources.appDistributionAccessIdentity).to.be.an('object');
    }
};
__decorate([
    mocha_typescript_1.timeout(60000),
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ServeCommandTest.prototype, "can build a project", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "can generate a serverless.yml in memory", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains functions", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml functions have correct data fields", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "frontend function has correct paths (/ and /*)", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains tables", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains cloudfront distribution", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains s3 bucket", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains s3 permission", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "parse yml does not contain null values for previously just undefined values", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ServeCommandTest.prototype, "yaml contains distribution access identity", null);
ServeCommandTest = __decorate([
    mocha_typescript_1.suite('Serverless::generate_yaml')
], ServeCommandTest);
//# sourceMappingURL=generate-yaml.js.map