"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const mocha_typescript_1 = require("mocha-typescript");
const codegen_1 = require("../../../lib/codegen");
let CodegenFunctionTest = class CodegenFunctionTest {
    'can be initialized'() {
        const gen = new codegen_1.Function('MyFunction');
        chai_1.expect(gen).to.be.an('object');
    }
    'can have untyped param'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.addParam('a');
        const code = gen.toString();
        chai_1.expect(code).to.contain('export default function MyFunction(a) {}');
    }
    'can have typed param'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.addParam('s', 'string');
        const code = gen.toString();
        chai_1.expect(code).to.contain('export default function MyFunction(s: string) {}');
    }
    'can have destructured param'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.addParam('{ children }');
        const code = gen.toString();
        chai_1.expect(code).to.contain('function MyFunction({ children }) {}');
    }
    'can have explicit return type'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.setReturnType('string');
        const code = gen.toString();
        chai_1.expect(code).to.contain('function MyFunction(): string {}');
    }
    'can have simple body written inline'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.setBodyText('return "hello world!"');
        const code = gen.toString();
        chai_1.expect(code).to.contain("{\n  return 'hello world!'\n}");
    }
    'can have complex body written incrementally'() {
        const gen = new codegen_1.Function('MyFunction');
        gen.setBodyComplex(writer => {
            writer
                .writeLine('const a = 3')
                .write('if (a === 5)')
                .block(() => {
                writer.writeLine('alert("fail!")');
            })
                .writeLine('else')
                .block(() => {
                writer.writeLine('alert("throw")');
            });
        });
        const code = gen.toString();
        chai_1.expect(code).to.contain("if (a === 5) {\n    alert('fail!')\n  }");
        chai_1.expect(code).to.contain("else {\n    alert('throw')\n  }\n");
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can be initialized", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have untyped param", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have typed param", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have destructured param", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have explicit return type", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have simple body written inline", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CodegenFunctionTest.prototype, "can have complex body written incrementally", null);
CodegenFunctionTest = __decorate([
    mocha_typescript_1.suite
], CodegenFunctionTest);
//# sourceMappingURL=function.js.map